@@ -1,6 +1,7 @@
 /* 
  *  at.c : Put file into atrun queue
  *  Copyright (C) 1993, 1994, 1995, 1996, 1997 Thomas Koenig
+ *  Copyright (C) 2002, 2005 Ryan Murray
  *
  *  Atrun & Atq modifications
  *  Copyright (C) 1993  David Parsons
@@ -17,7 +18,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -77,7 +78,7 @@
 #include "panic.h"
 #include "parsetime.h"
 #include "perm.h"
-
+#include "posixtm.h"
 #include "privs.h"
 
 /* Macros */
@@ -91,7 +92,6 @@
 #define SIZE 255
 
 #define TIMEFORMAT_POSIX	"%a %b %e %T %Y"
-#define TIMEFORMAT_ISO		"%Y-%m-%d %H:%M"
 #define TIMESIZE	50
 
 enum {
@@ -108,7 +108,6 @@ gid_t daemon_gid = (gid_t) - 3;
 
 /* File scope variables */
 
-static char rcsid[] = "$Id: at.c,v 1.29 1997/09/28 20:00:16 ig25 Exp $";
 char *no_export[] =
 {
     "TERM", "DISPLAY", "_", "SHELLOPTS", "BASH_VERSINFO", "EUID", "GROUPS", "PPID", "UID"
@@ -142,9 +141,18 @@ sigc(int signo)
 /* If the user presses ^C, remove the spool file and exit 
  */
     if (fcreated) {
-	PRIV_START
+	/*
+        PRIV_START
+
+        We need the unprivileged uid here since the file is owned by the real
+        (not effective) uid.
+        */
+        setregid(real_gid, effective_gid);
 	    unlink(atfile);
+        setregid(effective_gid, real_gid);
+        /*
 	PRIV_END
+        */
     }
     exit(EXIT_FAILURE);
 }
@@ -199,7 +207,7 @@ nextjob()
 	fscanf(fid, "%5lx", &jobno);
 	rewind(fid);
     } else {
-	fid = fopen(ATJOB_DIR "/.SEQ", "w");
+	fid = fopen(LFILE, "w");
 	if (fid == NULL)
 	    return EOF;
     }
@@ -231,6 +239,8 @@ writefile(time_t runtimer, char queue)
     pid_t pid;
     int istty;
     int kill_errno;
+    int rc;
+    int mailsize = 128;
 
 /* Install the signal handler for SIGINT; terminate after removing the
  * spool file if necessary
@@ -244,6 +254,16 @@ writefile(time_t runtimer, char queue)
 
     ppos = atfile + strlen(ATJOB_DIR) + 1;
 
+#ifdef _SC_LOGIN_NAME_MAX
+    errno = 0;
+    rc = sysconf(_SC_LOGIN_NAME_MAX);
+    if (rc > 0)
+	mailsize = rc;
+#else
+#  ifdef LOGIN_NAME_MAX
+    mailsize = LOGIN_NAME_MAX;
+#  endif
+#endif
     /* Loop over all possible file names for running something at this
      * particular time, see if a file is there; the first empty slot at any
      * particular time is used.  Lock the file LFILE first to make sure
@@ -292,14 +312,18 @@ writefile(time_t runtimer, char queue)
 	 * bit.  Yes, this is a kluge.
 	 */
 	cmask = umask(S_IRUSR | S_IWUSR | S_IXUSR);
+        seteuid(real_uid);
 	if ((fd = open(atfile, O_CREAT | O_EXCL | O_TRUNC | O_WRONLY, S_IRUSR)) == -1)
 	    perr("Cannot create atjob file %.500s", atfile);
+        seteuid(effective_uid);
 
 	if ((fd2 = dup(fd)) < 0)
 	    perr("Error in dup() of job file");
 
+        /*
 	if (fchown(fd2, real_uid, real_gid) != 0)
 	    perr("Cannot give away file");
+        */
 
     PRIV_END
 
@@ -331,19 +355,23 @@ writefile(time_t runtimer, char queue)
     mailname = getlogin();
     if (mailname == NULL)
 	mailname = getenv("LOGNAME");
-
-    if ((mailname == NULL) || (mailname[0] == '\0')
-	|| (strlen(mailname) > 8) || (getpwnam(mailname) == NULL)) {
+    if (mailname == NULL || mailname[0] == '\0' || getpwnam(mailname) == NULL) {
 	pass_entry = getpwuid(real_uid);
 	if (pass_entry != NULL)
 	    mailname = pass_entry->pw_name;
     }
+
+    if ((mailname == NULL) || (mailname[0] == '\0')
+	|| (strlen(mailname) > mailsize) ) {
+	panic("Cannot find username to mail output to");
+    }
     if (atinput != (char *) NULL) {
 	fpin = freopen(atinput, "r", stdin);
 	if (fpin == NULL)
 	    perr("Cannot open input file %.500s", atinput);
     }
-    fprintf(fp, "#!/bin/sh\n# atrun uid=%d gid=%d\n# mail %8s %d\n",
+
+    fprintf(fp, "#!/bin/sh\n# atrun uid=%d gid=%d\n# mail %s %d\n",
 	    real_uid, real_gid, mailname, send_mail);
 
     /* Write out the umask at the time of invocation
@@ -464,15 +492,7 @@ writefile(time_t runtimer, char queue)
 
     runtime = localtime(&runtimer);
 
-    /* We only use the sick POSIX time format if POSIXLY_CORRECT
-       is set.  Otherwise, we use ISO format.
-     */
-
-    if (getenv("POSIXLY_CORRECT") != NULL) {
-	strftime(timestr, TIMESIZE, TIMEFORMAT_POSIX, runtime);
-    } else {
-	strftime(timestr, TIMESIZE, TIMEFORMAT_ISO, runtime);
-    }
+    strftime(timestr, TIMESIZE, TIMEFORMAT_POSIX, runtime);
     fprintf(stderr, "job %ld at %s\n", jobno, timestr);
 
     /* Signal atd, if present. Usual precautions taken... */
@@ -525,7 +545,7 @@ writefile(time_t runtimer, char queue)
 }
 
 static void
-list_jobs()
+list_jobs(void)
 {
     /* List all a user's jobs in the queue, by looping through ATJOB_DIR, 
      * or everybody's if we are root
@@ -570,11 +590,8 @@ list_jobs()
 	runtimer = 60 * (time_t) ctm;
 	runtime = localtime(&runtimer);
 
-	if (getenv("POSIXLY_CORRECT") != NULL) {
-	    strftime(timestr, TIMESIZE, TIMEFORMAT_POSIX, runtime);
-	} else {
-	    strftime(timestr, TIMESIZE, TIMEFORMAT_ISO, runtime);
-	}
+	strftime(timestr, TIMESIZE, TIMEFORMAT_POSIX, runtime);
+
 	if ((pwd = getpwuid(buf.st_uid)))
 	  printf("%ld\t%s %c %s\n", jobno, timestr, queue, pwd->pw_name);
 	else
@@ -583,7 +600,7 @@ list_jobs()
     PRIV_END
 }
 
-static void
+static int
 process_jobs(int argc, char **argv, int what)
 {
     /* Delete every argument (job - ID) given
@@ -595,7 +612,11 @@ process_jobs(int argc, char **argv, int what)
     unsigned long ctm;
     char queue;
     long jobno;
+    int rc = EXIT_SUCCESS;
+    int done;
 
+    for (i = optind; i < argc; i++) {
+	done = 0;
     PRIV_START
 
     if (chdir(ATJOB_DIR) != 0)
@@ -618,7 +639,6 @@ process_jobs(int argc, char **argv, int what)
 	    if (sscanf(dirent->d_name, "%c%5lx%8lx", &queue, &jobno, &ctm) != 3)
 	    continue;
 
-	for (i = optind; i < argc; i++) {
 	    if (atoi(argv[i]) == jobno) {
 		if ((buf.st_uid != real_uid) && !(real_uid == 0)) {
 		    fprintf(stderr, "%s: Not owner\n", argv[i]);
@@ -627,15 +647,22 @@ process_jobs(int argc, char **argv, int what)
 		switch (what) {
 		case ATRM:
 
-		    PRIV_START
+                    /*
+                    We need the unprivileged uid here since the file is owned by the real
+                    (not effective) uid.
+                    */
+                    setregid(real_gid, effective_gid);
 
-			if (queue == '=') {
-			    fprintf(stderr, "Warning: deleting running job\n");
-			}
-			if (unlink(dirent->d_name) != 0)
-			    perr("Cannot unlink %.500s", dirent->d_name);
+		    if (queue == '=') {
+			fprintf(stderr, "Warning: deleting running job\n");
+		    }
+		    if (unlink(dirent->d_name) != 0) {
+			perr("Cannot unlink %.500s", dirent->d_name);
+			rc = EXIT_FAILURE;
+		    }
 
-		    PRIV_END
+                    setregid(effective_gid, real_gid);
+		    done = 1;
 
 		    break;
 
@@ -644,18 +671,20 @@ process_jobs(int argc, char **argv, int what)
 			FILE *fp;
 			int ch;
 
-			PRIV_START
+			setregid(real_gid, effective_gid);
+			fp = fopen(dirent->d_name, "r");
 
-			    fp = fopen(dirent->d_name, "r");
-
-			PRIV_END
-
-			if (!fp) {
-			    perr("Cannot open %s", dirent->d_name);
+			if (fp) {
+			    while ((ch = getc(fp)) != EOF) {
+				putchar(ch);
+			    }
+			    done = 1;
 			}
-			while ((ch = getc(fp)) != EOF) {
-			    putchar(ch);
+			else {
+			    perr("Cannot open %.500s", dirent->d_name);
+			    rc = EXIT_FAILURE;
 			}
+			setregid(effective_gid, real_gid);
 		    }
 		    break;
 
@@ -667,7 +696,13 @@ process_jobs(int argc, char **argv, int what)
 		}
 	    }
 	}
+	closedir(spool);
+	if (done != 1) {
+	    fprintf(stderr, "Cannot find jobid %s\n", argv[i] );
+	    rc = EXIT_FAILURE;
+	}
     }
+    return rc;
 }				/* delete_jobs */
 
 /* Global functions */
@@ -692,9 +727,9 @@ main(int argc, char **argv)
     char *pgm;
 
     int program = AT;		/* our default program */
-    char *options = "q:f:MmvldhVc";	/* default options for at */
+    char *options = "q:f:MmvldhVct:";	/* default options for at */
     int disp_version = 0;
-    time_t timer;
+    time_t timer = 0;
     struct passwd *pwe;
     struct group *ge;
 
@@ -764,6 +799,7 @@ main(int argc, char **argv)
 	    queue_set = 1;
 	    break;
 
+	case 'r':
 	case 'd':
 	    if (program != AT)
 		usage();
@@ -785,7 +821,7 @@ main(int argc, char **argv)
 		usage();
 
 	    program = BATCH;
-	    options = "f:q:mvV";
+	    options = "";
 	    break;
 
 	case 'V':
@@ -797,6 +833,15 @@ main(int argc, char **argv)
 	    options = "";
 	    break;
 
+	case 't':
+	    if (!posixtime(&timer, optarg, PDS_LEADING_YEAR | PDS_CENTURY | PDS_SECONDS)) {
+		fprintf(stderr, "invalid date format: %s\n", optarg);
+		exit(EXIT_FAILURE);
+	    }
+	    /* drop seconds */
+	    timer -= timer % 60;
+	    break;
+
 	default:
 	    usage();
 	    break;
@@ -804,39 +849,64 @@ main(int argc, char **argv)
     /* end of options eating
      */
 
-    if (disp_version)
+    if (disp_version) {
 	fprintf(stderr, "at version " VERSION "\n"
-	   "Bug reports to: ig25@rz.uni-karlsruhe.de (Thomas Koenig)\n");
+	   "Please report bugs to the Debian bug tracking system (http://bugs.debian.org/)\n"
+	   "or contact the maintainers (at@packages.debian.org).\n");
+	exit(EXIT_SUCCESS);
+    }
 
     /* select our program
      */
     if (!check_permission()) {
-	fprintf(stderr, "You do not have permission to use %s.\n", namep);
+	fprintf(stderr, "You do not have permission to use %.100s.\n", namep);
 	exit(EXIT_FAILURE);
     }
     switch (program) {
+	int i;
     case ATQ:
 
 	REDUCE_PRIV(daemon_uid, daemon_gid)
+
 	    list_jobs();
 	break;
 
     case ATRM:
 
 	REDUCE_PRIV(daemon_uid, daemon_gid)
-	    process_jobs(argc, argv, ATRM);
+	if (argc > optind) {
+	    for (i = optind; i < argc ; i++ )
+		if (strspn(argv[i],"0123456789") != strlen(argv[i])) {
+		    fprintf(stderr,"at: unknown jobid: %s\n", argv[i]);
+		    exit(EXIT_FAILURE);
+		}
+	    return process_jobs(argc, argv, ATRM);
+	}
+	else
+	    usage();
 	break;
 
     case CAT:
 
-	process_jobs(argc, argv, CAT);
+	if (argc > optind) {
+	    for (i = optind; i < argc ; i++ )
+		if (strspn(argv[i],"0123456789") != strlen(argv[i])) {
+		    fprintf(stderr,"at: unknown jobid: %s", argv[i]);
+		    exit(EXIT_FAILURE);
+		}
+	    return process_jobs(argc, argv, CAT);
+	}
+	else
+	    usage();
 	break;
 
     case AT:
 	if (argc > optind) {
+	    if (timer != 0) {
+                fprintf(stderr, "Cannot give time two times.\n");
+                exit(EXIT_FAILURE);
+            }
 	    timer = parsetime(argc - optind, argv + optind);
-	} else {
-	    timer = 0;
 	}
 
 	if (timer == 0) {
@@ -865,9 +935,13 @@ main(int argc, char **argv)
 	else
 	    queue = DEFAULT_BATCH_QUEUE;
 
-	if (argc > optind)
+	if (argc > optind) {
+            if (timer != 0) {
+                fprintf(stderr, "Cannot give time two times.\n");
+                exit(EXIT_FAILURE);
+            }
 	    timer = parsetime(argc, argv);
-	else
+        } else if (timer == 0)
 	    timer = time(NULL);
 
 	if (atverify) {
@@ -883,3 +957,4 @@ main(int argc, char **argv)
     }
     exit(EXIT_SUCCESS);
 }
+
