@@ -1,6 +1,7 @@
 /* 
  *  atd.c - run jobs queued by at; run with root privileges.
  *  Copyright (C) 1993, 1994, 1996 Thomas Koenig
+ *  Copyright (c) 2002, 2005 Ryan Murray
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 
@@ -22,18 +23,6 @@
 #include "config.h"
 #endif
 
-/* 
- * /usr/bin/mail aka /usr/bin/mailx require the subject to be
- * specified on the command line instead of reading it from stdin like
- * /usr/sbin/sendmail does. For now simply disable MAILC and MAILX,
- *
- * TODO: Remove tests for MAILC and MAILX from configure.in in the
- *       next upstream version.
- */
-#undef MAILC
-#undef MAILX
-
-
 /* System Headers */
 
 #include <sys/types.h>
@@ -110,7 +99,6 @@ gid_t daemon_gid = (gid_t) - 3;
 /* File scope variables */
 
 static char *namep;
-static char rcsid[] = "$Id: atd.c,v 1.28 1997/05/06 08:31:09 ig25 Exp $";
 static double load_avg = LOADAVG_MX;
 static time_t now;
 static time_t last_chg;
@@ -120,6 +108,25 @@ static int run_as_daemon = 0;
 
 static volatile sig_atomic_t term_signal = 0;
 
+#ifdef HAVE_PAM
+#include <security/pam_appl.h>
+
+static pam_handle_t *pamh = NULL;
+
+static const struct pam_conv conv = {
+	NULL
+};
+
+#define PAM_FAIL_CHECK if (retcode != PAM_SUCCESS) { \
+	fprintf(stderr,"\n%s\n",pam_strerror(pamh, retcode)); \
+	syslog(LOG_ERR,"%s",pam_strerror(pamh, retcode)); \
+	pam_end(pamh, retcode); exit(1); \
+    }
+#define PAM_END { retcode = pam_close_session(pamh,0); \
+		pam_end(pamh,retcode); }
+
+#endif /* HAVE_PAM */
+
 /* Signal handlers */
 RETSIGTYPE 
 set_term(int dummy)
@@ -132,6 +139,7 @@ RETSIGTYPE
 sdummy(int dummy)
 {
     /* Empty signal handler */
+    nothing_to_do = 0;
     return;
 }
 
@@ -196,8 +204,9 @@ run_file(const char *filename, uid_t uid, gid_t gid)
  */
     pid_t pid;
     int fd_out, fd_in;
-    char mailbuf[9], jobbuf[9];
+    char jobbuf[9];
     char *mailname = NULL;
+    int mailsize = 128;
     char *newname;
     FILE *stream;
     int send_mail = 0;
@@ -208,8 +217,26 @@ run_file(const char *filename, uid_t uid, gid_t gid)
     int nuid;
     int ngid;
     char queue;
+    char fmt[64];
     unsigned long jobno;
+#ifdef HAVE_PAM
+    int retcode;
+#endif
 
+#ifdef _SC_LOGIN_NAME_MAX
+    int rc;
+
+    errno = 0;
+    rc = sysconf(_SC_LOGIN_NAME_MAX);
+    if (rc > 0)
+	mailsize = rc;
+#else
+#  ifdef LOGIN_NAME_MAX
+    mailsize = LOGIN_NAME_MAX;
+#  endif
+#endif
+    if ((mailname = malloc(mailsize+1)) == NULL)
+	pabort("Job %8lu : out of virtual memory", jobno);
     sscanf(filename, "%c%5lx", &queue, &jobno);
 
     sprintf(jobbuf, "%8lu", jobno);
@@ -227,6 +254,8 @@ run_file(const char *filename, uid_t uid, gid_t gid)
      */
     if (link(filename, newname) == -1) {
 	if (errno == EEXIST) {
+	    free(mailname);
+	    free(newname);
 	    syslog(LOG_WARNING, "trying to execute job %.100s twice",filename);
 	    return;
 	} else {
@@ -243,6 +272,8 @@ run_file(const char *filename, uid_t uid, gid_t gid)
 	perr("Cannot fork");
 
     else if (pid != 0) {
+	free(mailname);
+	free(newname);
 	return;
     }
     /* Let's see who we mail to.  Hopefully, we can read it from
@@ -284,8 +315,8 @@ run_file(const char *filename, uid_t uid, gid_t gid)
 	     "aborting", jobno, filename);
 
     if (buf.st_nlink > 2) {
-	perr("Someboy is trying to run a linked script for job %8lu (%.500s)",
-	     filename);
+	perr("Somebody is trying to run a linked script for job %8lu (%.500s)",
+	     jobno, filename);
     }
     if ((fflags = fcntl(fd_in, F_GETFD)) < 0)
 	perr("Error in fcntl");
@@ -299,24 +330,22 @@ run_file(const char *filename, uid_t uid, gid_t gid)
      * NFS and works with local file systems.  It's not clear where
      * the bug is located.  -Joey
      */
-    if (fscanf(stream, "#!/bin/sh\n# atrun uid=%d gid=%d\n# mail %8s %d",
-	       &nuid, &ngid, mailbuf, &send_mail) != 4)
+    sprintf(fmt, "#!/bin/sh\n# atrun uid=%%d gid=%%d\n# mail %%%ds %%d",
+	mailsize );
+
+    if (fscanf(stream, fmt,
+	       &nuid, &ngid, mailname, &send_mail) != 4)
 	pabort("File %.500s is in wrong format - aborting",
 	       filename);
 
-    if (mailbuf[0] == '-')
-	pabort("illegal mail name %.300s in job %8lu (%.300s)", mailbuf,
+    if (mailname[0] == '-')
+	pabort("illegal mail name %.300s in job %8lu (%.300s)", mailname,
 	       jobno, filename);
 
-    mailname = mailbuf;
     if (nuid != uid)
 	pabort("Job %8lu (%.500s) - userid %d does not match file uid %d",
 	       jobno, filename, nuid, uid);
 
-    if (ngid != gid)
-	pabort("Job %8lu %.500s - groupid %d does not match file gid %d",
-	       jobno, filename, ngid, gid);
-
     /* We are now committed to executing this script.  Unlink the
      * original.
      */
@@ -348,6 +377,19 @@ run_file(const char *filename, uid_t uid, gid_t gid)
     fstat(fd_out, &buf);
     size = buf.st_size;
 
+#ifdef HAVE_PAM
+    PRIV_START
+    retcode = pam_start("atd", pentry->pw_name, &conv, &pamh);
+    PAM_FAIL_CHECK;
+    retcode = pam_acct_mgmt(pamh, PAM_SILENT);
+    PAM_FAIL_CHECK;
+    retcode = pam_open_session(pamh, PAM_SILENT);
+    PAM_FAIL_CHECK;
+    retcode = pam_setcred(pamh, PAM_ESTABLISH_CRED | PAM_SILENT);
+    PAM_FAIL_CHECK;
+    PRIV_END
+#endif
+
     close(STDIN_FILENO);
     close(STDOUT_FILENO);
     close(STDERR_FILENO);
@@ -363,7 +405,6 @@ run_file(const char *filename, uid_t uid, gid_t gid)
 	/* Set up things for the child; we want standard input from the
 	 * input file, and standard output and error sent to our output file.
 	 */
-
 	if (lseek(fd_in, (off_t) 0, SEEK_SET) < 0)
 	    perr("Error in lseek");
 
@@ -388,7 +429,7 @@ run_file(const char *filename, uid_t uid, gid_t gid)
 	    if (initgroups(pentry->pw_name, pentry->pw_gid))
 		perr("Cannot delete saved userids");
 
-	    if (setgid(gid) < 0)
+	    if (setgid(ngid) < 0)
 		perr("Cannot change group");
 
 	    if (setuid(uid) < 0)
@@ -412,6 +453,14 @@ run_file(const char *filename, uid_t uid, gid_t gid)
      */
     waitpid(pid, (int *) NULL, 0);
 
+#ifdef HAVE_PAM
+    PRIV_START
+	pam_setcred(pamh, PAM_DELETE_CRED | PAM_SILENT);
+	retcode = pam_close_session(pamh, PAM_SILENT);
+	pam_end(pamh, retcode);
+    PRIV_END
+#endif
+
     /* Send mail.  Unlink the output file after opening it, so it
      * doesn't hang around after the run.
      */
@@ -444,10 +493,6 @@ run_file(const char *filename, uid_t uid, gid_t gid)
 
 #if defined(SENDMAIL)
 	    execl(SENDMAIL, "sendmail", mailname, (char *) NULL);
-#elif  defined(MAILC)
-	    execl(MAILC, "mail", mailname, (char *) NULL);
-#elif  defined(MAILX)
-	    execl(MAILX, "mailx", mailname, (char *) NULL);
 #else
 #error      "No mail command specified."
 #endif
@@ -538,7 +583,7 @@ run_loop()
 	/* Skip lock files */
 	if (queue == '=') {
 	    if ((buf.st_nlink == 1) && (run_time + CHECK_INTERVAL <= now)) {
-		/* Remove stale lockfile */
+		/* Remove stale lockfile FIXME: lock the lockfile, if you fail, it's still in use. */
 		unlink(dirent->d_name);
 	    }
 	    continue;
@@ -574,7 +619,7 @@ run_loop()
 	nothing_to_do = 0;
 
 	/* There's a job for later.  Note its execution time if it's
-	 * the earlierst so far.
+	 * the earliest so far.
 	 */
 	if (run_time > now) {
 	    if (next_job > run_time) {
@@ -608,9 +653,15 @@ run_loop()
      */
     if (run_batch  && (next_batch <= now)) {
 	next_batch = now + batch_interval;
+#ifdef GETLOADAVG_PRIVILEGED
+	START_PRIV
+#endif
 	if (getloadavg(currlavg, 1) < 1) {
 	    currlavg[0] = 0.0;
 	}
+#ifdef GETLOADAVG_PRIVILEGED
+	END_PRIV
+#endif
 	if (currlavg[0] < load_avg) {
 	    run_file(batch_name, batch_uid, batch_gid);
 	    run_batch--;
@@ -628,10 +679,10 @@ run_loop()
 int
 main(int argc, char *argv[])
 {
-/* Browse through  ATJOB_DIR, checking all the jobfiles wether they should
+/* Browse through ATJOB_DIR, checking all the jobfiles whether they should
  * be executed and or deleted. The queue is coded into the first byte of
- * the job filename, the date (in minutes since Eon) as a hex number in the
- * following eight bytes, followed by a dot and a serial number.  A file
+ * the job filename, the next 5 bytes encode the serial number in hex, and
+ * the final 8 bytes encode the date (minutes since Eon) in hex.  A file
  * which has not been executed yet is denoted by its execute - bit set.
  * For those files which are to be executed, run_file() is called, which forks
  * off a child which takes care of I/O redirection, forks off another child
