@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,8 +48,6 @@
 #include "daemon.h"
 #include "privs.h"
 
-static char rcsid[] = "$Id: daemon.c,v 1.9 1996/08/12 08:23:55 ig25 Exp $";
-
 int daemon_debug;
 
 static int
@@ -133,20 +131,14 @@ daemon_setup()
 
     fd = open(PIDFILE, O_RDWR | O_CREAT | O_EXCL, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
 
-    PRIV_END
-
     if (fd == -1) {
 
 	if (errno != EEXIST)
 	    perr("Cannot open " PIDFILE);
 
-	PRIV_START
-
 	if ((fd = open(PIDFILE, O_RDWR)) < 0)
 	    perr("Cannot open " PIDFILE);
 
-	PRIV_END
-
 	fp = fdopen(fd, "rw");
 	if (fp == NULL) {
 	    perr("Cannot open " PIDFILE " for reading");
@@ -158,11 +150,7 @@ daemon_setup()
 
 	    syslog(LOG_NOTICE, "Removing stale lockfile for pid %d", pid);
 
-	    PRIV_START
-
-		rc = unlink(PIDFILE);
-
-	    PRIV_END
+	    rc = unlink(PIDFILE);
 
 	    if (rc == -1) {
 		perr("Cannot unlink " PIDFILE);
@@ -172,18 +160,16 @@ daemon_setup()
 	}
 	fclose(fp);
 
-	PRIV_START
-
 	unlink(PIDFILE);
 	fd = open(PIDFILE, O_RDWR | O_CREAT | O_EXCL,
 		  S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH);
 
-	PRIV_END
 
 	if (fd == -1)
 	    perr("Cannot open " PIDFILE " the second time round");
 
     }
+
     if (lock_fd(fd) == -1)
 	perr("Cannot lock " PIDFILE);
 
@@ -198,6 +184,8 @@ daemon_setup()
      */
     fflush(fp);
     fcntl(fd, F_SETFD, (long) 1);
+    PRIV_END
+
     return;
 }
 
